package org.xbib.interlibrary.action.order;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;

public abstract class AbstractOrderService implements OrderService {

    private static final Logger logger = Logger.getLogger(AbstractOrderService.class.getName());

    protected final ServiceArguments arguments;

    private final String name;

    private boolean enabled;

    private Throwable lastException;

    public AbstractOrderService(ServiceArguments arguments, String name) {
        this.arguments = arguments;
        this.name = arguments.getServiceSettings("name", name);
        this.enabled = arguments.getServiceSettingsAsBoolean("enabled", true);
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    @Override
    public boolean isEnabled() {
        return this.enabled;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public void addThrowable(Throwable throwable) {
        logger.log(Level.SEVERE, throwable.getMessage(), throwable);
        this.lastException = throwable;
    }

    @Override
    public Throwable getLastThrowable() {
        return lastException;
    }

    @Override
    public void execute(OrderRequest orderRequest, OrderResponseBuilder responseBuilder) {
        responseBuilder.setRequest(orderRequest);
    }

    @Override
    public int compareTo(Service<OrderRequest, OrderResponse, OrderResponseBuilder> service) {
        return getName().compareTo(service.getName());
    }
}
