package org.xbib.interlibrary.action.order;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import org.xbib.interlibrary.api.action.AbstractAction;
import org.xbib.interlibrary.api.action.Action;
import org.xbib.interlibrary.api.action.ActionArguments;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.action.Service;

public class OrderAction extends AbstractAction<OrderRequest, OrderResponse, OrderResponseBuilder, OrderService, OrderServiceProvider> {

    public static final OrderAction INSTANCE = new OrderAction();

    private static final Logger logger = Logger.getLogger(OrderAction.class.getName());

    private List<OrderService> services = new ArrayList<>();

    public OrderAction() {
    }

    @Override
    public Action<OrderRequest, OrderResponse, OrderResponseBuilder, OrderService> getInstance() {
        return INSTANCE;
    }

    @Override
    public void init(ActionArguments arguments) {
        this.services = createServices("action.order.service", arguments);
        logger.log(Level.INFO, () -> "services: " + services.size() + " " +
                services.stream().map(Service::getName).collect(Collectors.joining(",")));
    }

    @Override
    public List<OrderService> getServices() {
        return services;
    }

    @Override
    public OrderResponseBuilder newResponseBuilder(OrderRequest request) {
        return OrderResponse.builder().setRequest(request);
    }

    @Override
    public OrderResponseBuilder newEmptyResponseBuilder(OrderRequest request) {
        return OrderResponse.builder().setRequest(request).setStatus("global", Response.Status.EMPTY_RESPONSE);
    }

    @Override
    public OrderResponseBuilder newErrorResponse(OrderRequest request, Throwable throwable) {
        logger.log(Level.SEVERE, throwable.getMessage(), throwable);
        return OrderResponse.builder().setRequest(request).setStatus("global", Response.Status.ERROR).setThrowable(throwable);
    }
}
