package org.xbib.interlibrary.action.order;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.datastructures.api.Builder;
import org.xbib.datastructures.json.tiny.JsonBuilder;
import org.xbib.interlibrary.api.PackageInfo;
import org.xbib.interlibrary.api.action.AbstractResponse;
import org.xbib.interlibrary.api.util.ExceptionFormatter;

public class OrderResponse extends AbstractResponse<OrderRequest> {

    private static final Logger orderLogger = Logger.getLogger(OrderResponse.class.getPackageName());

    private final OrderResponseBuilder builder;

    public OrderResponse(OrderRequest request, Status status, OrderResponseBuilder builder) {
        super(request, status);
        this.builder = builder;
    }

    public static OrderResponseBuilder builder() {
        return new OrderResponseBuilder();
    }

    public void log() {
        if (orderLogger.isLoggable(Level.INFO)) {
            orderLogger.log(Level.INFO, getRequest() + " --> " + asJson());
        }
    }

    public String asJson() {
        try {
            Builder contentBuilder = JsonBuilder.builder();
            if (getThrowable() != null) {
                contentBuilder.beginMap();
                contentBuilder.field("version", PackageInfo.VERSION);
                if (getStatus() != null) {
                    contentBuilder.field("status", getStatus().name());
                }
                contentBuilder.field("exception", ExceptionFormatter.format(getThrowable()));
                contentBuilder.endMap();
            } else {
                buildResult(contentBuilder);
            }
            return contentBuilder.build();
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    private void buildResult(Builder contentBuilder) throws IOException {
        contentBuilder.beginMap();
        contentBuilder.field("version", PackageInfo.VERSION);
        if (getStatus() != null) {
            contentBuilder.field("status", getStatus().name());
        }
        if (builder.getThrowable() != null) {
            contentBuilder.field("exception", ExceptionFormatter.format(builder.getThrowable()));
        }
        // zero-based offset
        contentBuilder.fieldIfNotNull("offset", getRequest().getOffset());
        contentBuilder.fieldIfNotNull("size", getRequest().getSize());
        contentBuilder.endMap();
    }
}
